"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require("debug");
const authentication = require("feathers-authentication-client");
const hooks = require("feathers-hooks");
const localstorage = require("feathers-localstorage");
const socketio = require("feathers-socketio/client");
const feathers = require("feathers/client");
const storage = require("localstorage-memory");
const prom_client_1 = require("prom-client");
const log = debug("DS:Agent:Gateway");
class Gateway {
    constructor(eventEmitter, config) {
        this.eventEmitter = eventEmitter;
        this.config = config;
        this.connected = false;
        this.gauges = {};
        this.registry = new prom_client_1.Registry();
        this.authenticating = false;
        this.authenticated = false;
        log("Connecting to: %s", config.url);
        this.socket = require("socket.io-client")(config.url);
        this.socket.on("disconnect", () => {
            log("Disconnected from: %s", config.url);
            if (!this.reconnectInterval) {
                this.reconnectInterval = setInterval(this.socket.connect.bind(this.socket), 5000);
            }
        });
        this.socket.on("connect", () => {
            this.connected = true;
            this.eventEmitter.emit("gatewayConnect", this);
            log("Connected to: %s", config.url);
            this[this.authenticated ? "configure" : "authenticate"]().catch((error) => {
                log("Authentication/Configuration Error: %O", error);
            });
            if (this.reconnectInterval) {
                clearInterval(this.reconnectInterval);
            }
        });
        this.socket.on("fetchMetrics", (time, ack) => {
            ack(this.registry.metrics());
        });
        this.socket.on("reconnect", () => {
            log("Reconnected to: %s", config.url);
        });
        this.socket.on("error", (e) => log("error: %O", e));
        this.socket.on("connect_error", (e) => log("connection error: %O", e));
        this.socket.on("reconnect_error", (e) => log("reconnection error: %O", e));
        this.socket.on("connect_timeout", () => log("connection timeout"));
        this.storage = localstorage({ storage });
        if (config.events) {
            config.events.forEach((x) => {
                this.socket.on(x, (...args) => this.eventEmitter.emit(x, ...args, this));
            });
        }
        this.app = feathers()
            .configure(socketio(this.socket))
            .configure(hooks())
            .configure(authentication(Object.assign({ service: "agent", entity: "agent", storage }, config.auth)));
    }
    authenticate() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                log("Authenticating with: %s", this.config.url);
                if (this.authenticating || this.authenticated) {
                    return;
                }
                this.authenticating = true;
                const auth = {
                    strategy: "local",
                    type: "local",
                    name: "internal",
                    key: this.config.auth.key,
                };
                this.app.authenticate(auth)
                    .then((result) => {
                    this.authenticating = false;
                    this.eventEmitter.emit("gatewayAuthenticated", this);
                    log("Authenticated with: %s", this.config.url);
                    this.authToken = result.accessToken;
                    return this.configure();
                })
                    .then(resolve)
                    .catch((error) => {
                    this.authenticating = false;
                    reject(error);
                });
            });
        });
    }
    addGauge(nick, help, labelNames = []) {
        const name = (this.config.promPrefix || "discordservers_prod_bot_") + nick;
        this.gauges[nick] = new prom_client_1.Gauge({ name, help, labelNames, registers: [this.registry] });
    }
    emit(event, ...args) {
        this.socket.emit(event, ...args);
    }
    on(event, callback) {
        this.socket.on(event, callback);
    }
    configure() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                log("Configuring: %s", this.config.url);
                this.emit("configure", this.config, (configuration) => {
                    this.configuration = configuration;
                    if (this.config.commands) {
                        this.emit("registerCommands", Object.values(this.config.commands).map((x) => x.info), () => {
                            this.eventEmitter.emit("gatewayConfigured", this);
                            log("Configured: %s", this.config.url);
                            resolve();
                        });
                    }
                    else {
                        this.eventEmitter.emit("gatewayConfigured", this);
                        log("Configured: %s", this.config.url);
                        resolve();
                    }
                });
            });
        });
    }
}
exports.default = Gateway;
//# sourceMappingURL=Gateway.js.map